#!/bin/bash
# -*- mode: sh; indent-tabs-mode: nil; sh-basic-offset: 4; -*-
# vim: et sts=4 sw=4

#  SPDX-License-Identifier: LGPL-2.1+
#
#  Copyright © 2019-2021 Collabora Ltd.
#  Copyright © 2019-2021 Valve Corporation.
#
#  This file is part of steamos-customizations.
#
#  steamos-customizations is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the License,
#  or (at your option) any later version.

# Create udev rules for steamos partitions

. /lib/dracut-lib.sh

expand_dev() {
    local dev

    case "$1" in
    LABEL=*)
        dev="/dev/disk/by-label/${1#LABEL=}"
        ;;
    UUID=*)
        dev="${1#UUID=}"
        dev="/dev/disk/by-uuid/${dev,,}"
        ;;
    PARTUUID=*)
        dev="${1#PARTUUID=}"
        dev="/dev/disk/by-partuuid/${dev,,}"
        ;;
    PARTLABEL=*)
        dev="/dev/disk/by-partlabel/${1#PARTLABEL=}"
        ;;
    *)
        dev="$1"
        ;;
    esac

    echo "$dev"
}

# Main

info "Setting up SteamOS udev rules"

EFI=$(getarg 'rd.steamos.efi=')
if [ -z "$EFI" ]; then
    EFI=$(getarg 'steamos.efi=')
fi
if [ -z "$EFI" ]; then
    warn "No second loader found!"
    warn "Creates the by-partsets symlinks manually and exit."
    warn "*** Dropping you to a shell; the system will continue"
    warn "*** when you leave the shell."
    emergency_shell
    exit 0
fi

efidev="$(expand_dev "$EFI")"

info "Waiting for device ${efidev}"
wait_for_dev "${efidev}"

info "Mounting ${EFI##*=} with -o ro"
mount -o ro "${efidev}" /mnt 2>&1 | vinfo

trap "if ismounted /mnt; then umount /mnt; fi" 0
if ! ismounted /mnt; then
    warn "Mounting $efidev failed!"
    warn "*** Dropping you to a shell; the system will continue"
    warn "*** when you leave the shell."
    emergency_shell
fi

for partset in /mnt/SteamOS/partsets/*; do
    [ -e "$partset" ] || continue
    /usr/lib/steamos/steamos-partsets-generator "$partset"
done

udevadm control --reload-rules && udevadm trigger --settle
